<?php
namespace App\Factory;

use App\Exception;

class PaymentFactory {

    private $connection;

    public function __construct (\Doctrine\DBAL\Connection $connection) {
        $this->connection = $connection;
    }

    public function getReceiptType($cdFilial, $receipt, $alert = true) {
        //===================== Busca por filial =====================
        $sql = "
            SELECT CDTIPORECE
              
              FROM TIPORECEINTELOCAL TP
    
              JOIN INTEGRACAO INTE 
                ON TP.CDINTEGRACAO   = INTE.CDINTEGRACAO
    
             WHERE INTE.IDINTEGRACAO = :IDINTEGRACAO
               AND CDFILIAL          = :CDFILIAL
               AND UPPER(REPLACE(CDTPRECEINTE, ' ', '')) = :CDTPRECEINTE    
        ";

        $params = [
            ":IDINTEGRACAO" => 'ALP',
            ":CDFILIAL"     => $cdFilial,
            ":CDTPRECEINTE" => strtoupper(preg_replace("/\s+/", "", $receipt))
        ];

        $result = $this->connection->fetchAll($sql, $params);
        
        if(!empty($result) && isset($result[0]["CDTIPORECE"])) {
            return $result[0]["CDTIPORECE"];
        }
        //===================== Busca por filial =====================

        $sql = "
            SELECT CDTIPORECE

              FROM TIPORECEINTE TP
            
              JOIN INTEGRACAO INTE 
                ON TP.CDINTEGRACAO   = INTE.CDINTEGRACAO

             WHERE INTE.IDINTEGRACAO = :IDINTEGRACAO
               AND UPPER(REPLACE(CDTPRECEINTE, ' ', '')) = :CDTPRECEINTE
        ";
        
        $params = [
            ":CDTPRECEINTE" => strtoupper(preg_replace("/\s+/", "", $receipt)),
            ":IDINTEGRACAO" => 'ALP'
        ];
        
        $result = $this->connection->fetchAll($sql, $params);
        
        if(empty($result) && $alert) 
            throw new Exception\ParamsException("
                Não existe um tipo de recebimento mapeado no sistema para o código de pagamento {$receipt}.<br>
                <a href=''>Clique aqui</a> para consultar os códigos de pagamento suportados pelo Alphacode.
                Para mais detalhes sobre o mapeamento de tipos de recebimento, verifique a seção 'Parametrização da Integração', no manual da integração.");
        
        if(!isset($result[0]["CDTIPORECE"]) && $alert) 
            throw new Exception\ParamsException("
                Não existe um tipo de recebimento mapeado no sistema para o código de pagamento {$receipt}.<br>
                <a href=''>Clique aqui</a> para consultar os códigos de pagamento suportados pelo Alphacode.
                Para mais detalhes sobre o mapeamento de tipos de recebimento, verifique a seção 'Parametrização da Integração', no manual da integração.");
        
        if(empty($result)){
            return [];
        }
        
        return $result[0]["CDTIPORECE"];
    }
    
    public function getMoneyReceiptCode () {
        $sql = "
            SELECT MIN(CDTIPORECE) AS CDTIPORECE
              FROM TIPORECE
             WHERE IDTIPORECE = '4'
        ";
        
        $result = $this->connection->fetchAll($sql);
        return $result[0]["CDTIPORECE"];
    }
    
}
