<?php
namespace App\Factory;

use App\Exception;

class WebhookFactory {
    
    private $connection;
    
    public function __construct (\Doctrine\DBAL\Connection $connection) {
        $this->connection = $connection;
    }

    public function getTokenByToken($token) {
        $sql = "
            SELECT ID as idToken, TOKEN as token 
              FROM TOKEN 
             WHERE TOKEN = :TOKEN
        ";

        $params = [
            ':TOKEN' => $token
        ];

        return $this->connection->fetchAssoc($sql, $params);
    }
    
    public function getTokensByClient($clientId) {
        $sql = "
            SELECT ID as idToken, TOKEN as token 
              FROM TOKEN 
             WHERE IDCLIENTE = :IDCLIENTE
        ";

        $params = [
            ':IDCLIENTE' => $clientId
        ];

        return $this->connection->fetchAll($sql, $params);
    }

    public function getClientData($token) {
        $sql = "
            SELECT C.URL, C.NOME, C.TIPO, C.DBNAME
              
              FROM TOKEN T
              
              JOIN CLIENTE C
                ON T.IDCLIENTE = C.ID
             
             WHERE T.TOKEN = :TOKEN
          ORDER BY T.ID
        ";
        
        $params = [
            ":TOKEN" => $token    
        ];
        
        return $this->connection->fetchAssoc($sql, $params);
    }

    public function getMerchantsForDispatch() {
        $sql = "
            SELECT URL, NOME, TIPO, DBNAME
              
              FROM CLIENTE
        ";
        
        return $this->connection->fetchAll($sql);
    }
    
    public function getClientList() {
        $sql = "
            SELECT C.ID AS IDCLIENTE, C.NOME, C.URL, C.TIPO, C.DBNAME
              FROM CLIENTE C
          ORDER BY C.ID
        ";

        
        return $this->connection->fetchAll($sql);
    }
    
    public function addClient($nomeCliente, $urlCliente, $tipoCliente, $usuarioDb) {
        $sql = "INSERT INTO `CLIENTE`(`NOME`, `URL`, `TIPO`, `DBNAME`) VALUES (:NOME_CLIENTE, :URL_CLIENTE, :TIPO, :DBNAME)";

        $params = [
            ':NOME_CLIENTE' => $nomeCliente,
            ':URL_CLIENTE'  => $urlCliente,
            ':TIPO' => $tipoCliente,
            ':DBNAME'   => $usuarioDb
        ];

        $this->connection->executeQuery($sql, $params);
    }
    
    public function editClient($idCliente, $nomeCliente, $urlCliente, $tipoCliente, $usuarioDb) {

        $sql = "
            SELECT * 
              FROM CLIENTE 
            WHERE ID = :ID_CLIENTE
        ";

        $params = [
            ':ID_CLIENTE'   => $idCliente,
        ];
        
        $clientExists = $this->connection->fetchAssoc($sql, $params);
        
        if(empty($clientExists)) {
            throw new \Exception("Invalid client!");
        }

        $sql = "
            UPDATE CLIENTE 
            SET NOME            = :NOME_CLIENTE, 
                URL             = :URL_CLIENTE,
                TIPO            = :TIPO,
                DBNAME          = :DBNAME
            WHERE ID = :ID_CLIENTE
        ";

        $params = [
            ':ID_CLIENTE'   => $idCliente,
            ':NOME_CLIENTE' => $nomeCliente,
            ':URL_CLIENTE'  => $urlCliente,
            ':TIPO' => $tipoCliente,
            ':DBNAME' => $usuarioDb
        ];

        $this->connection->executeQuery($sql, $params);
    }

    public function deleteClient($idCliente) {
        $sql = "
            DELETE FROM TOKEN 
            WHERE IDCLIENTE = :ID_CLIENTE
        ";

        $params = [
            ':ID_CLIENTE'   => $idCliente
        ];

        $this->connection->executeQuery($sql, $params);

        $sql = "
            DELETE FROM CLIENTE 
            WHERE ID = :ID_CLIENTE
        ";

        $params = [
            ':ID_CLIENTE'   => $idCliente
        ];

        $this->connection->executeQuery($sql, $params);
    }

    public function getClientById($idCliente) {
        $sql = "
            SELECT ID, NOME, URL, TIPO, DBNAME 
              FROM CLIENTE 
             WHERE ID = :ID_CLIENTE 
        ";

        $params = [
            ':ID_CLIENTE' => $idCliente
        ];

        return $this->connection->fetchAssoc($sql, $params);
    }
    
    public function getClient($nomeCliente, $urlCliente) {
        $sql = "
            SELECT ID, NOME, URL, TIPO, DBNAME
              FROM CLIENTE 
             WHERE NOME = :NOME_CLIENTE 
               AND URL  = :URL_CLIENTE
        ";

        $params = [
            ':NOME_CLIENTE' => $nomeCliente,
            ':URL_CLIENTE'  => $urlCliente
        ];

        return $this->connection->fetchAssoc($sql, $params);
    }

    public function addToken($idCliente, $token) {
        $sql = "INSERT INTO `TOKEN`(`IDCLIENTE`, `TOKEN`) VALUES (:ID_CLIENTE, :TOKEN)";

        $params = [
            ':ID_CLIENTE' => $idCliente,
            ':TOKEN'      => $token
        ];

        $this->connection->executeQuery($sql, $params);
    }
    
    public function getToken($idCliente, $token) {
        $sql = "
            SELECT ID, IDCLIENTE, TOKEN 
              FROM TOKEN 
             WHERE IDCLIENTE = :ID_CLIENTE 
               AND TOKEN     = :TOKEN
        ";

        $params = [
            ':ID_CLIENTE'   => $idCliente,
            ':TOKEN'        => $token
        ];

        return $this->connection->fetchAssoc($sql, $params);
    }

    public function editToken($idToken, $token) {
        $sql = "
            UPDATE TOKEN 
               SET TOKEN = :TOKEN
             WHERE ID    = :ID_TOKEN
        ";

        $params = [
            ':ID_TOKEN' => $idToken,
            ':TOKEN'    => $token
        ];

        $this->connection->executeQuery($sql, $params);
    }

    public function getTokenById($idToken) {
        $sql = "
            SELECT ID, IDCLIENTE, TOKEN 
              FROM TOKEN 
             WHERE ID = :ID_TOKEN 
        ";

        $params = [
            ':ID_TOKEN' => $idToken
        ];

        return $this->connection->fetchAssoc($sql, $params);
    }
    
    public function deleteToken($idToken) {
        $sql = "
            DELETE FROM TOKEN 
            WHERE ID = :ID_TOKEN
        ";

        $params = [
            ':ID_TOKEN'   => $idToken
        ];

        $this->connection->executeQuery($sql, $params);
    }
    
    public function cleanToken() {
        $sql = "
            UPDATE PARAMGERAL 
            SET UBRTOKENV1='', UBRTOKENV2='', UBRTKNDTCRIAC=''
            WHERE ID = 1
        ";

        $params = [
        ];

        $this->connection->executeQuery($sql, $params);
    }
    
    public function updateParams($clientSecret, $clientId){
        $sql = "
            UPDATE PARAMGERAL
                SET CLIENTSECRET = :CLIENTSECRET,
                    CLIENTID     = :CLIENTID
            WHERE ID = 1
        ";
        
        $params = [
            ':CLIENTSECRET' => $clientSecret,
            ':CLIENTID'     => $clientId
        ];
        
        $this->connection->executeQuery($sql, $params);
    }
    
    public function getWebhookParams() {
        $sql = "
            SELECT CLIENTSECRET, CLIENTID, CURRENTTOKEN, TKNDTCRIAC
            FROM PARAMGERAL
            WHERE ID = 1
        ";
        
        $params = [];
        
        return $this->connection->fetchAssoc($sql, $params);
    }
    
    public function updateWebhookParams($token, $createdAt) {
        $sql = "
            UPDATE PARAMGERAL 
               SET CURRENTTOKEN = :CURRENTTOKEN,
                   TKNDTCRIAC   = :TKNDTCRIAC
             WHERE ID = 1
        ";
    
        $params = [
            ':CURRENTTOKEN' => $token,
            ':TKNDTCRIAC'   => $createdAt
        ];
    
        $this->connection->executeQuery($sql, $params);
    }

}
