<?php
namespace App\Util;

use App\Exception;

class Util {

    private $connection;

    const MERCHANT_API_BASEURL = "https://hm.merchant-api.alphacode.com.br";
    
    //TEKNISA SANDBOX API URL
    //const MERCHANT_API_BASEURL = "http://192.168.120.98:9091/odhen-ifood-bench/src/public/index.php/authentication/v1.0/oauth/token";
    
    public function __construct(\Doctrine\DBAL\Connection $connection) {
        $this->connection = $connection;
    }

    public function manipulateEventFile($type, $fileName, $content, $receivedFileName = null){

        $configFile = file_get_contents(__DIR__ . "/../../environment/config.json");
        $configData = json_decode($configFile, true);
        $basePath   = $configData['basePath'];

        if($type == "pending" && !file_exists("$basePath/events/ack/$fileName")) {

            $pendingPath = "$basePath/events/pending/$fileName";
            $receivedPath = "$basePath/events/received/$receivedFileName";

            if(!file_exists("$basePath/events/pending")) {
                mkdir("$basePath/events/pending", 0775, true);
            }

            if(!file_exists("$basePath/events/received")) {
                mkdir("$basePath/events/received", 0775, true);
            }

            file_put_contents($pendingPath, ($content . PHP_EOL));
            file_put_contents($receivedPath, ($content . PHP_EOL), FILE_APPEND);
        }

        if($type == "ack") {
            $pendingPath = "$basePath/events/pending/$fileName";

            if(file_exists($pendingPath)) {
                unlink($pendingPath);
            }
        }

        if($type == "order" && !file_exists("$basePath/events/order/$fileName")) {
            $pendingPath = "$basePath/events/pending/$fileName";
            $orderPath   = "$basePath/events/order/$fileName";

            if(!file_exists("$basePath/events/pending")) {
                mkdir("$basePath/events/pending", 0775, true);
            }

            if(!file_exists("$basePath/events/order")) {
                mkdir("$basePath/events/order", 0775, true);
            }

            file_put_contents($orderPath, ($content . PHP_EOL));
            unlink($pendingPath);
        }

        if($type == "cancelled" && !file_exists("$basePath/events/cancelled/$fileName")) {
            $pendingPath = "$basePath/events/pending/$fileName";
            $cancelledPath = "$basePath/events/cancelled/$fileName";

            if(!file_exists("$basePath/events/pending")) {
                mkdir("$basePath/events/pending", 0775, true);
            }

            if(!file_exists("$basePath/events/cancelled")) {
                mkdir("$basePath/events/cancelled", 0775, true);
            }

            file_put_contents($cancelledPath, ($content . PHP_EOL));
            unlink($pendingPath);
        }

        if($type == "cancelRequest" && !file_exists("$basePath/events/cancelRequest/$fileName")) {
            $isSelfCancelled = json_decode($content, true);
            $isSelfCancelled = $isSelfCancelled['metadata']['ownerName'];
            $pendingPath = "$basePath/events/pending/$fileName";

            if(!file_exists("$basePath/events/pending")) {
                mkdir("$basePath/events/pending", 0775, true);
            }

            if(strpos($isSelfCancelled, 'teknisa') === false) {
                $cancelRequestPath = "$basePath/events/cancelRequest/$fileName";

                if(!file_exists("$basePath/events/cancelRequest")) {
                    mkdir("$basePath/events/cancelRequest", 0775, true);
                }

                file_put_contents($cancelRequestPath, ($content . PHP_EOL));
            }
            
            unlink($pendingPath);
        }

        if($type == "processed") {
            $processedPath = "$basePath/events/processed/$fileName";
            $orderPath = "$basePath/events/order/$fileName";
            $cancelRequestPath = "$basePath/events/cancelRequest/$fileName";
            $cancelledPath = "$basePath/events/cancelled/$fileName";

            if(!file_exists("$basePath/events/processed")) {
                mkdir("$basePath/events/processed", 0775, true);
            }
            
            if(!file_exists("$basePath/events/order")) {
                mkdir("$basePath/events/order", 0775, true);
            }

            if(!file_exists("$basePath/events/cancelRequest")) {
                mkdir("$basePath/events/cancelRequest", 0775, true);
            }

            if(!file_exists("$basePath/events/cancelled")) {
                mkdir("$basePath/events/cancelled", 0775, true);
            }

            if(file_exists($orderPath)) {
                unlink($orderPath);
                file_put_contents($processedPath, ($content . PHP_EOL));
            }
            
            if(file_exists($cancelRequestPath)) {
                unlink($cancelRequestPath);
            }
            
            if(file_exists($cancelledPath)) {
                unlink($cancelledPath);
            }
        }
        
        if($type == "error") {
            $orderPath = "$basePath/events/order/$fileName";

            if(!file_exists("$basePath/events/order")) {
                mkdir("$basePath/events/order", 0775, true);
            }

            if(file_exists($orderPath)) {
                unlink($orderPath);
            }
        }
    }

    public function readEventFiles($type) {
        $configFile = file_get_contents(__DIR__ . "/../../environment/config.json");
        $configData = json_decode($configFile, true);
        $basePath   = $configData['basePath'];

        $events = [];

        if($type === "pending"){
            foreach (glob("$basePath/events/pending/*.json") as $event) {
                $data = json_decode(file_get_contents($event), true);
                $events[$data['orderId']] = $data;
            }
        }

        if($type === "order"){
            foreach (glob("$basePath/events/order/*.json") as $event) {
                $data = json_decode(file_get_contents($event), true);
                $events[$data['orderId']] = $data;
            }
        }

        if($type === "cancelRequest"){
            foreach (glob("$basePath/events/cancelRequest/*.json") as $event) {
                $data = json_decode(file_get_contents($event), true);
                $events[$data['orderId']] = $data;
            }
        }

        if($type === "cancelled"){
            foreach (glob("$basePath/events/cancelled/*.json") as $event) {
                $data = json_decode(file_get_contents($event), true);
                $events[$data['orderId']] = $data;
            }
        }

        return $events;
    }

    public function writeLog($type, $fileName, $content) {

        $configFile = file_get_contents(__DIR__ . "/../../environment/config.json");
        $configData = json_decode($configFile, true);
        $basePath   = $configData['basePath'];

        if($type == "requests") {
            $file = "$basePath/$fileName";
            file_put_contents($file, ($content . PHP_EOL), FILE_APPEND);
        }

        if($type == "general") {
            $folder = $basePath . '/general/';
            $folder .= date("Y") . "/" . date("m") . "/" . date("d") . "/";
            if(!file_exists($folder)) {
                mkdir($folder, 0775, true);
            }
            $file = $folder . 'general.txt';
            $text = date('d/m/Y H:i:s') . " - $content\r\n";

            file_put_contents($file, $text, FILE_APPEND);
        }

        if($type == "events") {
            $data   = json_decode($content, true);
            $folder = $basePath . "/events/";
    
            if(isset($data['meta']['user_id'])) {
                $folder .= $data['meta']['user_id'] . "/";    
            } else {
                $folder .= "undefined/";
            }
            
            $folder .= date("Y") . "/" . date("m") . "/" . date("d") . "/";
    
            if(!file_exists($folder)) {
                mkdir($folder, 0775, true);
            }
    
            $currentFile = $folder . date("H-i-s-u") . ".json";
            file_put_contents($currentFile, $content);
        }

        if($type == 'network') {
            $folder      = $basePath . "/networkError/" . date("Y") . "/" . date("m") . "/" . date("d") . "/";
            $currentFile = "$folder/$fileName";
            if(!file_exists($folder)) {
                mkdir($folder, 0775, true);
            }

            file_put_contents($currentFile, $content);
        }
        
        if($type == 'cancelled') {
            $folder      = $basePath . "/cancelled/" . date("Y") . "/" . date("m") . "/" . date("d") . "/";
            $currentFile = "$folder/$fileName";
            if(!file_exists($folder)) {
                mkdir($folder, 0775, true);
            }

            file_put_contents($currentFile, json_encode($content, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE));
        }
    }

    public function tofloat($num) {
        $dotPos = strrpos($num, '.');
        $commaPos = strrpos($num, ',');
        $sep = (($dotPos > $commaPos) && $dotPos) ? $dotPos : 
            ((($commaPos > $dotPos) && $commaPos) ? $commaPos : false);
       
        if (!$sep) {
            return floatval(preg_replace("/[^0-9]/", "", $num));
        } 
    
        $tmp = floatval(
            preg_replace("/[^0-9]/", "", substr($num, 0, $sep)) . '.' .
            preg_replace("/[^0-9]/", "", substr($num, $sep+1, strlen($num)))
        );
        
        $tmp2 = str_replace(".", ",", $tmp);

        if($this->getDbSchema() == 'en-us') {
            $tmp2 = str_replace(",", ".", $tmp2);
        }
        
        return $tmp2; 
    }
    
    public function request($url, $body, $header) {
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_CUSTOMREQUEST, 'POST');
        curl_setopt($ch, CURLOPT_HTTPHEADER, $header);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $body);
        curl_setopt($ch, CURLOPT_HTTP_VERSION, CURL_HTTP_VERSION_1_1);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_TIMEOUT, 300);
        curl_setopt($ch, CURLOPT_FAILONERROR, true);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
        
        $response = curl_exec($ch);
        $httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        
        if (!empty(curl_error($ch)) || (stripos($response, "<html>"))) {
            $fileName = date("H-i-s") . ".txt";
            $this->writeLog('network', $fileName, $this->formatNetworkError([
                'eventTime' => date("d/m/Y H:i:s"),
                'header'    => $header,
                'url'       => $url,
                'method'    => 'POST',
                'params'    => $body,
                'httpCode'  => $httpCode,
                'error'     => curl_error($ch),
                'response'  => $response
            ]));

            throw new \Exception("Erro na comunicação de chaveamento.");
        }
        
        curl_close($ch);
        return $response;
    }
    
    public function authenticate($clientId, $clientSecret) {
        $url    = self::MERCHANT_API_BASEURL . "/authentication/v1.0/oauth/token";
        $method = "POST";
        $header = [
            "Content-type: application/x-www-form-urlencoded"
        ];

        $params = http_build_query([
            "clientSecret" => $clientSecret,
            "clientId"     => $clientId,
            "grantType"    => "client_credentials"
        ]);

        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_CUSTOMREQUEST, $method);
        curl_setopt($ch, CURLOPT_HTTPHEADER, $header);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $params);
        curl_setopt($ch, CURLOPT_HTTP_VERSION, CURL_HTTP_VERSION_1_1);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_TIMEOUT, 15);
        curl_setopt($ch, CURLOPT_FAILONERROR, true);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);

        $response = curl_exec($ch);
        $httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);

        if (!empty(curl_error($ch)) || (stripos($response, "<html>"))) {
            $fileName = date("H-i-s") . ".txt";
            $this->writeLog('network', $fileName, $this->formatNetworkError([
                'eventTime' => date("d/m/Y H:i:s"),
                'header'    => $header,
                'url'       => $url,
                'method'    => $method,
                'params'    => $params,
                'httpCode'  => $httpCode,
                'error'     => curl_error($ch),
                'response'  => $response
            ]));

            throw new \Exception("Erro na comunicação com o iFood.");
        }

        curl_close($ch);
        return $response;
    }

    public function formatNetworkError($networkData) {
        $eventTime = $networkData['eventTime'];
        $header    = implode($networkData['header'], "\r\n");
        $url       = $networkData['url'];
        $method    = $networkData['method'];
        $params    = is_array($networkData['params']) ? implode($networkData['params'], "\r\n") : $networkData['params'];
        $httpCode  = $networkData['httpCode'];
        $error     = $networkData['error'];
        $response  = $networkData['response'];

        $template = file_get_contents(__DIR__ . '/../../templates/networkError.txt');
        $templateParams = [
            ':EVENT'     => $eventTime,
            ':HEADER'    => $header,
            ':URL'       => $url,
            ':METHOD'    => $method,
            ':PARAMS'    => $params,
            ':HTTP_CODE' => $httpCode,
            ':ERROR'     => $error,
            ':RESPONSE'  => $response            
        ];

        return str_replace(array_keys($templateParams), array_values($templateParams), $template);
    }
}